# Generated by Django 5.0.1 on 2024-01-28 22:11

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):
    initial = True

    dependencies = [
        ("artd_location", "0001_initial"),
        ("artd_partner", "0001_initial"),
    ]

    operations = [
        migrations.CreateModel(
            name="Customer",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "created_at",
                    models.DateTimeField(
                        auto_now_add=True,
                        help_text="Date time on which the object was created.",
                        verbose_name="Created at",
                    ),
                ),
                (
                    "updated_at",
                    models.DateTimeField(
                        auto_now=True,
                        help_text="Date time on which the object was last updated.",
                        verbose_name="Updated at",
                    ),
                ),
                (
                    "status",
                    models.BooleanField(
                        default=True,
                        help_text="Status of the object.",
                        verbose_name="Status",
                    ),
                ),
                (
                    "name",
                    models.CharField(
                        blank=True,
                        help_text="Name of the customer.",
                        max_length=100,
                        verbose_name="Name",
                    ),
                ),
                (
                    "last_name",
                    models.CharField(
                        blank=True,
                        help_text="Last name of the customer.",
                        max_length=100,
                        null=True,
                        verbose_name="Last name",
                    ),
                ),
                (
                    "birth_date",
                    models.DateField(
                        blank=True,
                        help_text="Birth date of the customer.",
                        null=True,
                        verbose_name="Birth date",
                    ),
                ),
                (
                    "document",
                    models.CharField(
                        blank=True,
                        help_text="Customer document",
                        max_length=50,
                        null=True,
                        verbose_name="Customer document",
                    ),
                ),
                (
                    "email",
                    models.EmailField(
                        blank=True,
                        help_text="Email of the customer.",
                        max_length=100,
                        null=True,
                        verbose_name="Email",
                    ),
                ),
                (
                    "phone",
                    models.CharField(
                        blank=True,
                        help_text="Phone of the customer.",
                        max_length=100,
                        null=True,
                        verbose_name="Phone",
                    ),
                ),
                (
                    "other_data",
                    models.JSONField(
                        blank=True,
                        help_text="Other customer data",
                        null=True,
                        verbose_name="Other data",
                    ),
                ),
                (
                    "city",
                    models.ForeignKey(
                        blank=True,
                        help_text="City of the customer.",
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        to="artd_location.city",
                        verbose_name="City",
                    ),
                ),
            ],
            options={
                "verbose_name": "Customer",
                "verbose_name_plural": "Customers",
            },
        ),
        migrations.CreateModel(
            name="CustomerAdditionalFields",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "created_at",
                    models.DateTimeField(
                        auto_now_add=True,
                        help_text="Date time on which the object was created.",
                        verbose_name="Created at",
                    ),
                ),
                (
                    "updated_at",
                    models.DateTimeField(
                        auto_now=True,
                        help_text="Date time on which the object was last updated.",
                        verbose_name="Updated at",
                    ),
                ),
                (
                    "status",
                    models.BooleanField(
                        default=True,
                        help_text="Status of the object.",
                        verbose_name="Status",
                    ),
                ),
                (
                    "name",
                    models.CharField(
                        help_text="Field name",
                        max_length=250,
                        verbose_name="Field name",
                    ),
                ),
                (
                    "field_type",
                    models.CharField(
                        choices=[
                            ("number", "Number"),
                            ("date", "Date"),
                            ("email", "Email"),
                            ("password", "Password"),
                            ("text", "Text"),
                        ],
                        help_text="Field type",
                        max_length=10,
                        verbose_name="Field type",
                    ),
                ),
                (
                    "label",
                    models.CharField(
                        help_text="Field label",
                        max_length=50,
                        verbose_name="Field label",
                    ),
                ),
                (
                    "required",
                    models.BooleanField(
                        default=False,
                        help_text="Is required?",
                        verbose_name="Is required?",
                    ),
                ),
                (
                    "partner",
                    models.ForeignKey(
                        help_text="Partner of the field.",
                        on_delete=django.db.models.deletion.CASCADE,
                        to="artd_partner.partner",
                        verbose_name="Partner",
                    ),
                ),
            ],
            options={
                "verbose_name": "Customer Additional Field",
                "verbose_name_plural": "Customer Additional Fields",
            },
        ),
        migrations.CreateModel(
            name="CustomerAddress",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "created_at",
                    models.DateTimeField(
                        auto_now_add=True,
                        help_text="Date time on which the object was created.",
                        verbose_name="Created at",
                    ),
                ),
                (
                    "updated_at",
                    models.DateTimeField(
                        auto_now=True,
                        help_text="Date time on which the object was last updated.",
                        verbose_name="Updated at",
                    ),
                ),
                (
                    "status",
                    models.BooleanField(
                        default=True,
                        help_text="Status of the object.",
                        verbose_name="Status",
                    ),
                ),
                (
                    "phone",
                    models.CharField(
                        blank=True,
                        help_text="Customer phone",
                        max_length=50,
                        null=True,
                        verbose_name="Phone",
                    ),
                ),
                (
                    "address",
                    models.TextField(
                        blank=True,
                        help_text="Customer address",
                        null=True,
                        verbose_name="Customer address",
                    ),
                ),
                (
                    "other_data",
                    models.JSONField(
                        blank=True,
                        help_text="Other customer address data",
                        null=True,
                        verbose_name="Other data",
                    ),
                ),
                (
                    "city",
                    models.ForeignKey(
                        help_text="City of the customer.",
                        on_delete=django.db.models.deletion.CASCADE,
                        to="artd_location.city",
                        verbose_name="City",
                    ),
                ),
                (
                    "customer",
                    models.ForeignKey(
                        help_text="Customer",
                        on_delete=django.db.models.deletion.CASCADE,
                        to="artd_customer.customer",
                        verbose_name="Customer",
                    ),
                ),
            ],
            options={
                "verbose_name": "Customer Address",
                "verbose_name_plural": "Customer Addresss",
            },
        ),
        migrations.CreateModel(
            name="Tag",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "created_at",
                    models.DateTimeField(
                        auto_now_add=True,
                        help_text="Date time on which the object was created.",
                        verbose_name="Created at",
                    ),
                ),
                (
                    "updated_at",
                    models.DateTimeField(
                        auto_now=True,
                        help_text="Date time on which the object was last updated.",
                        verbose_name="Updated at",
                    ),
                ),
                (
                    "status",
                    models.BooleanField(
                        default=True,
                        help_text="Status of the object.",
                        verbose_name="Status",
                    ),
                ),
                (
                    "description",
                    models.CharField(
                        help_text="Tag description",
                        max_length=250,
                        verbose_name="Tag description",
                    ),
                ),
                (
                    "partner",
                    models.ForeignKey(
                        help_text="Partner of the tag.",
                        on_delete=django.db.models.deletion.CASCADE,
                        to="artd_partner.partner",
                        verbose_name="Partner",
                    ),
                ),
            ],
            options={
                "verbose_name": "Tag",
                "verbose_name_plural": "Tags",
            },
        ),
        migrations.CreateModel(
            name="CustomerTag",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "created_at",
                    models.DateTimeField(
                        auto_now_add=True,
                        help_text="Date time on which the object was created.",
                        verbose_name="Created at",
                    ),
                ),
                (
                    "updated_at",
                    models.DateTimeField(
                        auto_now=True,
                        help_text="Date time on which the object was last updated.",
                        verbose_name="Updated at",
                    ),
                ),
                (
                    "status",
                    models.BooleanField(
                        default=True,
                        help_text="Status of the object.",
                        verbose_name="Status",
                    ),
                ),
                (
                    "customer",
                    models.ForeignKey(
                        help_text="Customer",
                        on_delete=django.db.models.deletion.CASCADE,
                        to="artd_customer.customer",
                        verbose_name="Customer",
                    ),
                ),
                (
                    "tag",
                    models.ForeignKey(
                        help_text="Tag.",
                        on_delete=django.db.models.deletion.CASCADE,
                        to="artd_customer.tag",
                        verbose_name="Customer tag",
                    ),
                ),
            ],
            options={
                "verbose_name": "Customer Tag",
                "verbose_name_plural": "Customer Tags",
            },
        ),
    ]



from django.contrib import admin
from artd_customer.models import (
    Customer,
    Tag,
    CustomerTag,
    CustomerAddress,
    CustomerAdditionalFields,
    CustomerGroup,
    CustomerGroupChangeLog,
)
from django_json_widget.widgets import JSONEditorWidget
from django.db import models


@admin.register(Customer)
class CustomerAdmin(admin.ModelAdmin):
    search_fields = [
        "name",
        "phone",
        "email",
        "partner__name",
    ]
    list_display = [
        "id",
        "name",
        "phone",
        "email",
        "status",
    ]

    def has_delete_permission(self, request, obj=None):
        return False

    formfield_overrides = {
        models.JSONField: {"widget": JSONEditorWidget},
    }


@admin.register(Tag)
class TagAdmin(admin.ModelAdmin):
    search_fields = [
        "description",
    ]
    list_display = [
        "description",
    ]


@admin.register(CustomerTag)
class CustomerTagAdmin(admin.ModelAdmin):
    search_fields = [
        "customer",
        "tag",
    ]
    list_display = [
        "customer",
        "tag",
    ]


@admin.register(CustomerAddress)
class CustomerAddressAdmin(admin.ModelAdmin):
    search_fields = [
        "customer__email",
        "city__name",
        "phone",
        "address",
    ]
    list_display = [
        "customer",
        "city",
        "phone",
        "address",
    ]
    formfield_overrides = {
        models.JSONField: {"widget": JSONEditorWidget},
    }


@admin.register(CustomerAdditionalFields)
class CustomerAdditionalFieldsAdmin(admin.ModelAdmin):
    search_fields = [
        "partner__name",
        "name",
        "field_type",
        "label",
        "required",
    ]
    list_display = [
        "partner",
        "name",
        "field_type",
        "label",
        "required",
    ]

@admin.register(CustomerGroup)
class CustomerGroupAdmin(admin.ModelAdmin):
    search_fields = [
        "group_code",
        "group_name",
        "status",
        "created_at",
        "updated_at",
    ]
    list_display = [
        "group_code",
        "group_name",
        "status",
        "created_at",
        "updated_at",
    ]
    readonly_fields = [
        "created_at",
        "updated_at",
    ]

@admin.register(CustomerGroupChangeLog)
class CustomerGroupChangeLogAdmin(admin.ModelAdmin):
    search_fields = [
        "customer",
        "old_group",
        "new_group",
    ]
    list_display = [
        "customer",
        "old_group",
        "new_group",
        "status",
        "created_at",
        "updated_at",
    ]
    readonly_fields = [
        "customer",
        "old_group",
        "new_group",
        "status",
        "created_at",
        "updated_at",
    ]


"""
 * Copyright (C) ArtD SAS - All Rights Reserved
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 * Written by Jonathan Favian Urzola Maldonado <jonathan@artd.com.co>, 2023
"""

from django.db import models
from django.utils.translation import gettext_lazy as _
from artd_location.models import City
from artd_partner.models import Partner

FIELD_TYPES = [
    ("number", _("Number")),
    ("date", _("Date")),
    ("email", _("Email")),
    ("password", _("Password")),
    ("text", _("Text")),
]


class CustomerBaseModel(models.Model):
    created_at = models.DateTimeField(
        _("Created at"),
        help_text=_("Date time on which the object was created."),
        auto_now_add=True,
        editable=False,
    )
    updated_at = models.DateTimeField(
        _("Updated at"),
        help_text=_("Date time on which the object was last updated."),
        auto_now=True,
        editable=False,
    )
    status = models.BooleanField(
        _("Status"),
        help_text=_("Status of the object."),
        default=True,
    )
    source = models.JSONField(
        _("Source"),
        help_text=_("Source of the object."),
        blank=True,
        null=True,
    )
    external_id = models.CharField(
        _("External ID"),
        help_text=_("External ID of the object."),
        max_length=100,
        blank=True,
        null=True,
    )

    class Meta:
        abstract = True


class CustomerGroup(CustomerBaseModel):
    """Model definition for Customer Group."""

    group_code = models.SlugField(
        _("Customer code"),
        max_length=50,
        help_text=_("Customer code"),
        unique=True,
    )
    group_name = models.CharField(
        _("Customer group name"),
        max_length=100,
        help_text=_("Customer group name"),
    )
    group_description = models.TextField(
        _("Customer group description"),
        help_text=_("Customer group description"),
        blank=True,
        null=True,
    )

    class Meta:
        """Meta definition for Customer Group."""

        verbose_name = "Customer Group"
        verbose_name_plural = "Customer Groups"

    def __str__(self):
        """Unicode representation of Customer Group."""
        return self.group_name


class Customer(CustomerBaseModel):
    name = models.CharField(
        _("Name"),
        help_text=_("Name of the customer."),
        max_length=100,
        blank=True,
    )
    last_name = models.CharField(
        _("Last name"),
        help_text=_("Last name of the customer."),
        max_length=100,
        blank=True,
        null=True,
    )
    birth_date = models.DateField(
        _("Birth date"),
        help_text=_("Birth date of the customer."),
        blank=True,
        null=True,
    )
    document = models.CharField(
        _("Customer document"),
        max_length=50,
        help_text=_("Customer document"),
        blank=True,
        null=True,
    )
    email = models.EmailField(
        _("Email"),
        help_text=_("Email of the customer."),
        max_length=100,
        blank=True,
        null=True,
    )
    phone = models.CharField(
        _("Phone"),
        help_text=_("Phone of the customer."),
        max_length=100,
        blank=True,
        null=True,
    )
    city = models.ForeignKey(
        City,
        verbose_name=_("City"),
        help_text=_("City of the customer."),
        on_delete=models.CASCADE,
        blank=True,
        null=True,
    )
    customer_group = models.ForeignKey(
        CustomerGroup,
        verbose_name=_("Customer group"),
        help_text=_("Customer group."),
        on_delete=models.CASCADE,
        blank=True,
        null=True,
    )

    other_data = models.JSONField(
        _("Other data"),
        help_text=_("Other customer data"),
        blank=True,
        null=True,
    )

    class Meta:
        verbose_name = _("Customer")
        verbose_name_plural = _("Customers")

    def __str__(self):
        return self.name

    def save(self, *args, **kwargs):
        if not self._state.adding:  # Verificar si se está actualizando
            # Obtener el valor anterior del campo antes de guardar
            old_model = Customer.objects.get(pk=self.pk)

        else:
            old_model = None  # Es un nuevo registro

        super(Customer, self).save(*args, **kwargs)

        if old_model is None:
            news_group = CustomerGroup.objects.get(group_code="new")
            CustomerGroupChangeLog.objects.create(
                customer=self,
                old_group=news_group,
                new_group=news_group,
            )
        else:
            if old_model.customer_group != self.customer_group:
                CustomerGroupChangeLog.objects.create(
                    customer=self,
                    old_group=old_model.customer_group,
                    new_group=self.customer_group,
                )


class Tag(CustomerBaseModel):
    partner = models.ForeignKey(
        Partner,
        verbose_name=_("Partner"),
        help_text=_("Partner of the tag."),
        on_delete=models.CASCADE,
    )

    description = models.CharField(
        _("Tag description"),
        max_length=250,
        help_text=_("Tag description"),
    )

    class Meta:
        verbose_name = _("Tag")
        verbose_name_plural = _("Tags")

    def __str__(self):
        return self.description


class CustomerTag(CustomerBaseModel):
    customer = models.ForeignKey(
        Customer,
        verbose_name=_("Customer"),
        help_text=_("Customer"),
        on_delete=models.CASCADE,
    )
    tag = models.ForeignKey(
        Tag,
        verbose_name=_("Customer tag"),
        help_text=_("Tag."),
        on_delete=models.CASCADE,
    )

    class Meta:
        verbose_name = "Customer Tag"
        verbose_name_plural = "Customer Tags"

    def __str__(self):
        return self.customer.name + " - " + self.tag.description


class CustomerAddress(CustomerBaseModel):
    customer = models.ForeignKey(
        Customer,
        verbose_name=_("Customer"),
        help_text=_("Customer"),
        on_delete=models.CASCADE,
    )
    city = models.ForeignKey(
        City,
        verbose_name=_("City"),
        help_text=_("City of the customer."),
        on_delete=models.CASCADE,
    )
    phone = models.CharField(
        _("Phone"),
        max_length=50,
        help_text=_("Customer phone"),
        blank=True,
        null=True,
    )
    address = models.TextField(
        _("Customer address"),
        help_text=_("Customer address"),
        blank=True,
        null=True,
    )
    other_data = models.JSONField(
        _("Other data"),
        help_text=_("Other customer address data"),
        blank=True,
        null=True,
    )

    class Meta:
        """Meta definition for CustomerAddress."""

        verbose_name = _("Customer Address")
        verbose_name_plural = _("Customer Addresss")

    def __str__(self):
        """Unicode representation of CustomerAddress."""
        return f"{self.customer} {self.address}"


class CustomerAdditionalFields(CustomerBaseModel):
    partner = models.ForeignKey(
        Partner,
        verbose_name=_("Partner"),
        help_text=_("Partner of the field."),
        on_delete=models.CASCADE,
    )
    name = models.CharField(
        "Field name",
        help_text="Field name",
        max_length=250,
    )
    field_type = models.CharField(
        _("Field type"),
        help_text=_("Field type"),
        max_length=10,
        choices=FIELD_TYPES,
    )
    label = models.CharField(
        _("Field label"),
        max_length=50,
        help_text=_("Field label"),
    )
    required = models.BooleanField(
        _("Is required?"),
        help_text=_("Is required?"),
        default=False,
    )

    class Meta:
        """Meta definition for CustomerAdditionalFields."""

        verbose_name = _("Customer Additional Field")
        verbose_name_plural = _("Customer Additional Fields")

    def __str__(self):
        """Unicode representation of CustomerAdditionalFields."""
        return f"{self.name}"


class CustomerGroupChangeLog(CustomerBaseModel):
    """Model definition for Customer Group Change Log."""

    customer = models.ForeignKey(
        Customer,
        verbose_name=_("Customer"),
        help_text=_("Customer"),
        on_delete=models.CASCADE,
    )

    old_group = models.ForeignKey(
        CustomerGroup,
        verbose_name=_("Old group"),
        help_text=_("Old group"),
        on_delete=models.CASCADE,
        related_name="old_group",
    )

    new_group = models.ForeignKey(
        CustomerGroup,
        verbose_name=_("New group"),
        help_text=_("New group"),
        on_delete=models.CASCADE,
        related_name="new_group",
    )

    class Meta:
        """Meta definition for Customer Group Change Log."""

        verbose_name = "Customer Group Change Log"
        verbose_name_plural = "Customer Group Change Logs"

    def __str__(self):
        """Unicode representation of Customer Group Change Log."""
        return str(self.id)



# Generated by Django 5.0.1 on 2024-02-14 11:09

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("artd_customer", "0001_initial"),
    ]

    operations = [
        migrations.CreateModel(
            name="CustomerGroup",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "created_at",
                    models.DateTimeField(
                        auto_now_add=True,
                        help_text="Date time on which the object was created.",
                        verbose_name="Created at",
                    ),
                ),
                (
                    "updated_at",
                    models.DateTimeField(
                        auto_now=True,
                        help_text="Date time on which the object was last updated.",
                        verbose_name="Updated at",
                    ),
                ),
                (
                    "status",
                    models.BooleanField(
                        default=True,
                        help_text="Status of the object.",
                        verbose_name="Status",
                    ),
                ),
                (
                    "group_code",
                    models.SlugField(
                        help_text="Customer code",
                        unique=True,
                        verbose_name="Customer code",
                    ),
                ),
                (
                    "group_name",
                    models.CharField(
                        help_text="Customer group name",
                        max_length=100,
                        verbose_name="Customer group name",
                    ),
                ),
                (
                    "group_description",
                    models.TextField(
                        blank=True,
                        help_text="Customer group description",
                        null=True,
                        verbose_name="Customer group description",
                    ),
                ),
            ],
            options={
                "verbose_name": "Customer Group",
                "verbose_name_plural": "Customer Groups",
            },
        ),
        migrations.AddField(
            model_name="customer",
            name="customer_group",
            field=models.ForeignKey(
                blank=True,
                help_text="Customer group.",
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                to="artd_customer.customergroup",
                verbose_name="Customer group",
            ),
        ),
        migrations.CreateModel(
            name="CustomerGroupChangeLog",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "created_at",
                    models.DateTimeField(
                        auto_now_add=True,
                        help_text="Date time on which the object was created.",
                        verbose_name="Created at",
                    ),
                ),
                (
                    "updated_at",
                    models.DateTimeField(
                        auto_now=True,
                        help_text="Date time on which the object was last updated.",
                        verbose_name="Updated at",
                    ),
                ),
                (
                    "status",
                    models.BooleanField(
                        default=True,
                        help_text="Status of the object.",
                        verbose_name="Status",
                    ),
                ),
                (
                    "customer",
                    models.ForeignKey(
                        help_text="Customer",
                        on_delete=django.db.models.deletion.CASCADE,
                        to="artd_customer.customer",
                        verbose_name="Customer",
                    ),
                ),
                (
                    "new_group",
                    models.ForeignKey(
                        help_text="New group",
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="new_group",
                        to="artd_customer.customergroup",
                        verbose_name="New group",
                    ),
                ),
                (
                    "old_group",
                    models.ForeignKey(
                        help_text="Old group",
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="old_group",
                        to="artd_customer.customergroup",
                        verbose_name="Old group",
                    ),
                ),
            ],
            options={
                "verbose_name": "Customer Group Change Log",
                "verbose_name_plural": "Customer Group Change Logs",
            },
        ),
    ]
